import { routerRedux } from 'dva/router';
import { accountLogin, accountLogout } from '../services/user';
import { setAuthority, setLocalUser } from '../utils/authority';
import { reloadAuthorized } from '../utils/Authorized';

export default {
  namespace: 'login',

  state: {
    status: undefined,
  },

  effects: {
    *login({ payload }, { call, put }) {
      const response = yield call(accountLogin, payload);
      yield put({
        type: 'changeLoginStatus',
        payload: response,
      });
      // Login successfully
      if (response.success) {
        reloadAuthorized();
        yield put(routerRedux.push('/dashboard/analysis'));
      }
    },
    *logout(_, { call, put }) {
      const response = yield call(accountLogout);
      if (response.success) {
        reloadAuthorized();
        yield put(routerRedux.push('/user/login'));
      }
    },
  },

  reducers: {
    changeLoginStatus(state, { payload }) {
      if (payload.data) {
        setLocalUser(payload.data);
        setAuthority(payload.data.id === '1' ? 'root' : 'general');
      }
      return {
        ...state,
        status: payload.success,
        repMsg: payload.message,
      };
    },
  },
};
