import { message } from 'antd';
import { routerRedux } from 'dva/router';
import {
  queryMerchantList,
  queryMerchantItem,
  createMerchantItem,
  updateMerchantItem,
  deleteMerchantItem,
  depositMerchantItem,
  queryMerchantRecommend,
  updateMerchantRecommend,
} from '../services/merchant';

export default {
  namespace: 'merchant',

  state: {
    list: [],
    pageNo: 1,
    pageSize: 15,
    totalSize: 0,
    currentItem: {},
    recommendList: [],
  },

  effects: {
    *fetchMerchantList({ payload }, { call, put }) {
      const response = yield call(queryMerchantList, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: response.data.list || [],
            pageSize: response.data.pageSize,
            totalSize: response.data.totalSize,
            pageNo: response.data.pageNo,
          },
        });
      }
    },
    *fetchMerchantItem({ payload }, { call, put }) {
      const response = yield call(queryMerchantItem, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            currentItem: response.data || {},
          },
        });
      }
    },
    *createMerchantItem({ payload, state }, { call, put }) {
      const response = yield call(createMerchantItem, payload);
      if (response.success) {
        message.success('创建商户成功');
        yield put(routerRedux.push({
          state,
          pathname: '/merchant/list',
        }));
      }
    },
    *deleteMerchantItem({ payload, states }, { call, put }) {
      const response = yield call(deleteMerchantItem, payload);
      if (response.success) {
        message.success('删除商户成功');
        yield put({
          type: 'fetchMerchantList',
          payload: states.Queryers,
        });
      }
    },
    *updateMerchantItem({ payload, states }, { call, put }) {
      const response = yield call(updateMerchantItem, payload);
      if (response.success) {
        message.success('修改商户成功');
        yield put(routerRedux.push({
          pathname: '/merchant/list',
          state: states,
        }));
      }
    },
    *depositMerchantItem({ payload, states }, { call, put }) {
      const response = yield call(depositMerchantItem, payload);
      if (response.success) {
        message.success('账户充值成功');
        yield put(routerRedux.push({
          pathname: '/merchant/list',
          state: states,
        }));
      }
    },
    *queryMerchantRecommend({ payload }, { call, put }) {
      const response = yield call(queryMerchantRecommend, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            recommendList: response.data || [],
          },
        });
      }
    },
    *updateMerchantRecommend({ payload, states }, { call, put }) {
      const response = yield call(updateMerchantRecommend, payload);
      if (response.success) {
        message.success('推荐位列表更新成功');
        yield put(routerRedux.push({
          pathname: '/frontend/recommend/list',
          state: states,
        }));
      }
    },
  },

  reducers: {
    querySuccess(state, action) {
      return {
        ...state,
        ...action.payload,
      };
    },
    fixRecommendList(state, action) {
      return {
        ...state,
        recommendList: action.payload,
      };
    },
    cleanItemState(state) {
      return {
        ...state,
        currentItem: {},
      };
    },
  },
};
