import React, { Component } from 'react';
import pathToRegexp from 'path-to-regexp';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { message, Form, Table, Modal, Card, Button, Input, Switch, Radio } from 'antd';
import { statusToBool, boolToStatus } from '../../../utils/utils';
import RBDragSortTable from '../../../components/RBDragSortTable/index';
import Selector from '../../../components/RBTableSelector/Selector';
import FooterToolbar from '../../../components/FooterToolbar/index';
import styles from './TagCreate.less';

const Message = message;

const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 2 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 14 },
    md: { span: 12 },
  },
};

@connect(({ loading, tag, tagGroup, shelves }) => ({
  tag,
  shelves,
  tagGroup,
  submitting: loading.models.tag,
  sLoading: loading.models.shelves,
  tLoading: loading.models.tagGroup,
}))
@Form.create()
export default class TagCreatePage extends Component {
  state = {
    tagGroupSelectorDestroy: true,
    shelvesSelectorDestroy: true,
    productType: 'Course',
  };
  componentWillMount() {
    const match = pathToRegexp('/frontend/tag/create').exec(this.props.location.pathname);
    if (match) {
      this.cleanPageState();
    }
  }
  componentDidMount() {
    const matchId = this.isEdit();
    if (matchId) {
      this.props.dispatch({
        type: 'tag/fetchTagItem',
        payload: { tagId: matchId },
      });
    }
  }
  isEdit = () => {
    const { location } = this.props;
    const match = pathToRegexp('/frontend/tag/edit/:id').exec(location.pathname);
    if (match) {
      return match[1];
    }
    return false;
  }
  cleanPageState=() => {
    this.props.dispatch({
      type: 'tag/cleanItemState',
      payload: {},
    });
  }
  handleRadioChange=(e) => {
    this.setState({ productType: e.target.value });
    this.selectorDataFetcher(e.target.value, {});
  }
  selectorDataFetcher=(name, params) => {
    switch (name) {
      case 'Course':
        this.props.dispatch({
          type: 'shelves/fetchCourseItemList',
          payload: params,
        });
        break;
      case 'Support':
        this.props.dispatch({
          type: 'shelves/fetchSupportItemList',
          payload: params,
        });
        break;
      case 'TagGroup':
        this.props.dispatch({
          type: 'tagGroup/fetchTagGroupList',
          payload: params,
        });
        break;
      default:
        break;
    }
  }
  handleTagGroupSelectorModalShow = () => {
    this.setState({
      tagGroupSelectorDestroy: false,
    });
    this.selectorDataFetcher('TagGroup');
  }
  handleShelvesSelectorModalShow = () => {
    const { tag } = this.props;
    const { currentItem } = tag;
    const { merchantId } = currentItem;
    if (!merchantId) {
      Message.error('请先选择标签所属的标签组!');
      return;
    }
    const params = { merchantId };
    this.setState({
      shelvesSelectorDestroy: false,
    });
    this.selectorDataFetcher(this.state.productType, params);
  }
  handleTagGroupSelectorChange = (params) => {
    this.selectorDataFetcher('TagGroup', params);
  }
  handleShelvesSelectorChange = (params) => {
    const { tag } = this.props;
    const { currentItem } = tag;
    const { merchantId } = currentItem;
    this.selectorDataFetcher(this.state.productType, { ...params, merchantId });
  }
  handleTagGroupSelectorCancel = () => {
    this.setState({
      tagGroupSelectorDestroy: true,
    });
  }
  handleShelvesSelectorCancel = () => {
    this.setState({
      shelvesSelectorDestroy: true,
    });
  }
  handleTagGroupSelectorFinish = (rows) => {
    this.setState({
      tagGroupSelectorDestroy: true,
    });
    if (!rows || !rows.length) {
      return;
    }
    const { id, name, merchantName, merchantId } = rows[0];
    this.props.dispatch({
      type: 'tag/fixCurrentItem',
      payload: {
        merchantId,
        merchantName,
        groupId: id,
        groupName: name,
      },
    });
  }
  handleShelvesSelectorFinish = (rows) => {
    this.setState({
      shelvesSelectorDestroy: true,
    });
    if (!rows || !rows.length) {
      return;
    }
    this.props.dispatch({
      type: 'tag/fixCurrentItem',
      payload: { productList: rows },
    });
  }
  handleDragSortTableChange = (rows) => {
    this.props.dispatch({
      type: 'tag/fixCurrentItem',
      payload: { productList: rows },
    });
  }
  handlePageBack = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/frontend/tag',
      state: this.props.location.state,
    }));
  }
  handlePageSubmit = (e) => {
    e.preventDefault();
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (!err) {
        const { name, status } = values;
        const { tag } = this.props;
        const { currentItem } = tag;
        const { productList = [], groupId } = currentItem;
        const id = this.isEdit();
        if (id) {
          this.props.dispatch({
            type: 'tag/updateTagItem',
            payload: {
              id,
              name,
              groupId,
              status: boolToStatus(status),
              productList: productList.map(item => item.pid),
            },
            states: this.props.location.state,
          });
        } else {
          this.props.dispatch({
            type: 'tag/createTagItem',
            payload: {
              name,
              groupId,
              status: boolToStatus(status),
              productList: productList.map(item => item.pid),
            },
            states: this.props.location.state,
          });
        }
      }
    });
  }

  render() {
    const { form, submitting, tLoading, sLoading, shelves, tag, tagGroup } = this.props;
    const { shelvesSelectorDestroy, tagGroupSelectorDestroy, productType } = this.state;
    const { currentItem } = tag;
    const { status, name, groupName, merchantName, productList = [] } = currentItem;
    const { getFieldDecorator } = form;

    const tagGroupData = [{
      fieldName: '标签组名称',
      value: groupName,
      key: 'groupName',
    }, {
      fieldName: '渠道名称',
      value: merchantName,
      key: 'merchantName',
    }];
    const tagGroupColumns = [{
      dataIndex: 'fieldName',
      key: 1,
      width: '30%',
    }, {
      dataIndex: 'value',
      key: 2,
      width: '70%',
    }];
    const productColumns = [{
      title: '产品编号',
      dataIndex: 'code',
      key: 1,
      width: '35%',
    }, {
      title: '产品名称',
      dataIndex: 'name',
      key: 2,
    }];
    const renderProductModalTitle = () => {
      return (
        <Radio.Group
          value={productType}
          onChange={this.handleRadioChange}
          className={styles.radio}
        >
          <Radio.Button value="Course">课程</Radio.Button>
          <Radio.Button value="Support">配套</Radio.Button>
        </Radio.Group>
      );
    };
    const getTagGroupModal = () => {
      return (
        <Modal
          width={1100}
          footer={null}
          visible
          title="标签组"
          maskClosable={false}
          onCancel={this.handleTagGroupSelectorCancel}
        >
          <Selector
            multiple={false}
            loading={tLoading}
            selectorName="TagGroup"
            list={tagGroup.list}
            pageNo={tagGroup.pageNo}
            pageSize={tagGroup.pageSize}
            totalSize={tagGroup.totalSize}
            onCancel={this.handleTagGroupSelectorCancel}
            onChange={this.handleTagGroupSelectorChange}
            onFinish={this.handleTagGroupSelectorFinish}
          />
        </Modal>
      );
    };
    const getProductModal = () => {
      return (
        <Modal
          width={1100}
          footer={null}
          visible
          title={renderProductModalTitle()}
          maskClosable={false}
          onCancel={this.handleShelvesSelectorCancel}
        >
          <Selector
            multiple
            loading={sLoading}
            selectorName={productType}
            list={shelves.list}
            pageNo={shelves.pageNo}
            pageSize={shelves.pageSize}
            totalSize={shelves.totalSize}
            onCancel={this.handleShelvesSelectorCancel}
            onChange={this.handleShelvesSelectorChange}
            onFinish={this.handleShelvesSelectorFinish}
          />
        </Modal>
      );
    };

    const renderTagGroupCardName = () => {
      return (
        <div className={styles.cardName}>
          <span>
            <a disabled={!!this.isEdit()} onClick={this.handleTagGroupSelectorModalShow}>所属标签组</a>
          </span>
        </div>
      );
    };
    const renderProductCardName = () => {
      return (
        <div className={styles.cardName}>
          <span>
            <a onClick={this.handleShelvesSelectorModalShow}>上架产品</a>
          </span>
        </div>
      );
    };
    return (
      <div>
        {/* 基础信息Card */}
        <Card title="基础信息" style={{ marginBottom: 16 }}>
          <Form>
            <Form.Item hasFeedback label="标签名称" {...formItemLayout}>
              {getFieldDecorator('name', {
                rules: [{ required: true, message: '请填写标签名称' }],
                initialValue: name,
              })(
                <Input placeholder="请输入" />
              )}
            </Form.Item>
            <Form.Item label="标签状态" {...formItemLayout}>
              {getFieldDecorator('status', {
                valuePropName: 'checked',
                initialValue: statusToBool(status),
              })(
                <Switch checkedChildren="正常" unCheckedChildren="删除" />
              )}
            </Form.Item>
          </Form>
        </Card>
        {/* 标签组选择Card */}
        <Card title={renderTagGroupCardName()} style={{ marginBottom: 16 }}>
          <Table
            bordered
            showHeader={false}
            pagination={false}
            columns={tagGroupColumns}
            dataSource={tagGroupData}
          />
          {!tagGroupSelectorDestroy && getTagGroupModal()}
        </Card>
        {/* 产品选择Card */}
        <Card title={renderProductCardName()} style={{ marginBottom: 70 }}>
          <RBDragSortTable
            data={productList}
            columns={productColumns}
            onChange={this.handleDragSortTableChange}
          />
          {!shelvesSelectorDestroy && getProductModal()}
        </Card>
        <FooterToolbar style={{ width: '100%' }}>
          <Button
            onClick={this.handlePageBack}
            style={{ marginRight: 10 }}
          >取消
          </Button>
          <Button
            type="primary"
            loading={submitting}
            onClick={this.handlePageSubmit}
          >提交
          </Button>
        </FooterToolbar>
      </div>
    );
  }
}
