import React, { Component } from 'react';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import { Card, Modal, Button, message } from 'antd';
import PictureCardList from './PictureCardList';
import PictureTableList from './PictureTableList';
import PageHeaderLayout from '../../../layouts/PageHeaderLayout';

const Message = message;

@connect(({ resource, loading }) => ({
  resource,
  loading: loading.models.resource,
}))
export default class PictureListPage extends Component {
  constructor(props) {
    super(props);
    const { state } = props.location;
    this.state = {
      UIParams: (state || {}).UIParams, // 组件的状态参数
      Queryers: (state || {}).Queryers, // 查询的条件参数
      isCard: true,
    };
  }
  componentDidMount() {
    this.props.dispatch({
      type: 'resource/fetchImageList',
      payload: { ...this.state.Queryers },
    });
  }
  handleShowTypeChange = () => {
    this.setState({ isCard: !this.state.isCard });
  }
  // 增加
  handleCreateOperation = () => {
    this.props.dispatch(
      routerRedux.push({
        pathname: '/resource/picture-upload/single',
        state: this.state,
      })
    );
  }
  // 删除
  handleDeleteOperation = (item) => {
    Modal.confirm({
      okText: '确定',
      cancelText: '取消',
      title: '您确定要删除这张图片吗?',
      onOk: () => {
        this.props.dispatch({
          type: 'resource/deleteImage',
          payload: { id: item.id },
        });
      },
    });
  }
  // 修改
  handleEditOperation = (data) => {
    this.props.dispatch(
      routerRedux.push({
        pathname: `/resource/picture-edit/${data.id}`,
        state: {
          ...this.state,
          ...data,
        },
      })
    );
  }
  // 查询
  handleFilterOperation = (params, states) => {
    this.props.dispatch({
      type: 'resource/fetchImageList',
      payload: params,
    });
    this.setState({
      UIParams: states,
      Queryers: params,
    });
  }
  // 批量
  handleBatchOperation = () => {
    Message.info('暂不支持批量操作!');
  }

  render() {
    const { resource, loading } = this.props;
    const { list, totalSize, pageSize, pageNo } = resource;
    const publicProps = {
      loading,
      pageNo,
      pageSize,
      totalSize,
      dataSource: list,
      onCreateClick: this.handleCreateOperation,
      onDeleteClick: this.handleDeleteOperation,
      onEditClick: this.handleEditOperation,
      onFilterClick: this.handleFilterOperation,
      onBatchClick: this.handleBatchOperation,
    };
    return (
      <PageHeaderLayout>
        <Card>
          <Button.Group style={{ marginBottom: 16 }}>
            <Button
              style={{ width: 50 }}
              onClick={this.handleShowTypeChange}
              icon="appstore-o"
              type={this.state.isCard ? 'primary' : null}
            />
            <Button
              style={{ width: 50 }}
              onClick={this.handleShowTypeChange}
              icon="table"
              type={this.state.isCard ? null : 'primary'}
            />
          </Button.Group>
          {this.state.isCard ?
            <PictureCardList {...publicProps} /> :
            <PictureTableList {...publicProps} />
          }
        </Card>
      </PageHeaderLayout>
    );
  }
}
