import { stringify } from 'qs';
import request from '../utils/request';
import { api, RESOURCE_IMAGE, RESOURCE_VIDEO, PAGE_SIZE } from '../utils/config';
import { getSignature } from '../utils/signature';

export async function queryImageResource(params) {
  const newParams = {
    pageSize: PAGE_SIZE,
    ...params,
    type: RESOURCE_IMAGE,
  };
  return request(`${api.resource}?${stringify(newParams)}`);
}

export async function queryVideoResource(params) {
  const newParams = {
    pageSize: PAGE_SIZE,
    ...params,
    type: RESOURCE_VIDEO,
  };
  return request(`${api.resource}?${stringify(newParams)}`);
}

export async function queryOssSignature(params) {
  const signature = getSignature();
  const expireTime = Math.floor(((new Date()).getTime() / 1000) + 5).toString();

  // 检查本地签名是否过期
  if (signature.expire >= expireTime) {
    return signature;
  }
  // 过期后请求新的签名
  return request(`${api.signature}?${stringify(params)}`);
}

export async function createImage(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.picture}`, options);
}

export async function deleteImage(params) {
  const options = {
    method: 'DELETE',
  };
  return request(`${api.picture}/${params.id}`, options);
}

export async function updateImage(params) {
  const options = {
    method: 'PUT',
    body: params,
  };
  return request(`${api.picture}`, options);
}
